@@ -29,7 +29,7 @@
  * |   provided "as is" without express or implied warranty.		|
  * +-------------------------------------------------------------------+
  *
- * RCS: @(#) $Id: tkImgGIF.c,v 1.24.2.1 2004/07/27 20:31:02 das Exp $
+ * RCS: @(#) $Id: tkImgGIF.c,v 1.24.2.6 2008/02/01 16:59:58 rmax Exp $
  */
 
 /*
@@ -55,6 +55,7 @@
 
 typedef struct mFile {
     unsigned char *data;	/* mmencoded source string */
+    int length;			/* Length of string in bytes */
     int c;			/* bits left over from previous character */
     int state;			/* decoder state (0-4 or GIF_DONE) */
 } MFile;
@@ -173,7 +174,7 @@ static int		Mread _ANSI_ARGS_((unsigned char *dst, size_t size,
 static int		Mgetc _ANSI_ARGS_((MFile *handle));
 static int		char64 _ANSI_ARGS_((int c));
 static void		mInit _ANSI_ARGS_((unsigned char *string,
-			    MFile *handle));
+			    int length, MFile *handle));
 
 
 /*
@@ -242,7 +243,7 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
     int srcX, srcY;		/* Coordinates of top-left pixel to be used
 				 * in image being read. */
 {
-    int fileWidth, fileHeight;
+    int fileWidth, fileHeight, imageWidth, imageHeight;
     int nBytes, index = 0, argc = 0, i;
     Tcl_Obj **objv;
     Tk_PhotoImageBlock block;
@@ -375,8 +376,8 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	    goto error;
 	}
 
-	fileWidth = LM_to_uint(buf[4],buf[5]);
-	fileHeight = LM_to_uint(buf[6],buf[7]);
+	imageWidth = LM_to_uint(buf[4],buf[5]);
+	imageHeight = LM_to_uint(buf[6],buf[7]);
 
 	bitPixel = 1<<((buf[8]&0x07)+1);
 
@@ -418,8 +419,8 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	     * of the less frequent case, I chose to maintain high
 	     * performance for the common case.
 	     */
-	    if (ReadImage(interp, (char *) trashBuffer, chan, fileWidth,
-		    fileHeight, colorMap, 0, 0, 0, 0, 0, -1) != TCL_OK) {
+	    if (ReadImage(interp, (char *) trashBuffer, chan, imageWidth,
+		    imageHeight, colorMap, 0, 0, 0, 0, 0, -1) != TCL_OK) {
 		goto error;
 	    }
 	    continue;
@@ -440,8 +441,8 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	    srcX = 0;
 	}
 
-	if (width > fileWidth) {
-	    width = fileWidth;
+	if (width > imageWidth) {
+	    width = imageWidth;
 	}
 
 	index = LM_to_uint(buf[2],buf[3]);
@@ -450,8 +451,8 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	    destY -= srcY; height += srcY;
 	    srcY = 0;
 	}
-	if (height > fileHeight) {
-	    height = fileHeight;
+	if (height > imageHeight) {
+	    height = imageHeight;
 	}
 
 	if ((width <= 0) || (height <= 0)) {
@@ -463,12 +464,12 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	block.height = height;
 	block.pixelSize = (transparent>=0) ? 4 : 3;
 	block.offset[3] = (transparent>=0) ? 3 : 0;
-	block.pitch = block.pixelSize * fileWidth;
-	nBytes = block.pitch * fileHeight;
+	block.pitch = block.pixelSize * imageWidth;
+	nBytes = block.pitch * imageHeight;
 	block.pixelPtr = (unsigned char *) ckalloc((unsigned) nBytes);
 
-	if (ReadImage(interp, (char *) block.pixelPtr, chan, fileWidth,
-		fileHeight, colorMap, fileWidth, fileHeight, srcX, srcY,
+	if (ReadImage(interp, (char *) block.pixelPtr, chan, imageWidth,
+		imageHeight, colorMap, fileWidth, fileHeight, srcX, srcY,
 		BitSet(buf[8], INTERLACE), transparent) != TCL_OK) {
 	    goto error;
 	}
@@ -552,7 +553,7 @@ StringMatchGIF(dataObj, format, widthPtr, heightPtr, interp)
 	/*
 	 * Try interpreting the data as Base64 encoded
 	 */
-	mInit((unsigned char *) data, &handle);
+	mInit((unsigned char *) data, length, &handle);
 	got = Mread(header, 10, 1, &handle);
 	if (got != 10
 		|| ((strncmp(GIF87a, (char *) header, 6) != 0)
@@ -599,7 +600,7 @@ StringReadGIF(interp, dataObj, format, imageHandle,
     int width, height;		/*   image to copy */
     int srcX, srcY;
 {
-    int result;
+    int result, length;
     MFile handle;
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
@@ -609,14 +610,14 @@ StringReadGIF(interp, dataObj, format, imageHandle,
     /*
      * Check whether the data is Base64 encoded
      */
-    data = (char *) Tcl_GetByteArrayFromObj(dataObj, NULL);
+    data = (char *) Tcl_GetByteArrayFromObj(dataObj, &length);
     if ((strncmp(GIF87a, data, 6) != 0) && (strncmp(GIF89a, data, 6) != 0)) {
-	mInit((unsigned char *)data, &handle);
+	mInit((unsigned char *)data, length, &handle);
 	tsdPtr->fromData = 1;
 	dataSrc = (Tcl_Channel) &handle;
     } else {
 	tsdPtr->fromData = 2;
-	mInit((unsigned char *)data, &handle);
+	mInit((unsigned char *)data, length, &handle);
 	dataSrc = (Tcl_Channel) &handle;
     }
     result = FileReadGIF(interp, dataSrc, "inline data",
@@ -825,6 +826,12 @@ ReadImage(interp, imagePtr, chan, len, rows, cmap,
 		Tcl_PosixError(interp), (char *) NULL);
 	return TCL_ERROR;
     }
+
+    if (initialCodeSize > MAX_LWZ_BITS) {
+	Tcl_SetResult(interp, "malformed image", TCL_STATIC);
+	return TCL_ERROR;
+    }
+
     if (transparent != -1) {
 	cmap[transparent][CM_RED] = 0;
 	cmap[transparent][CM_GREEN] = 0;
@@ -994,7 +1001,7 @@ ReadImage(interp, imagePtr, chan, len, rows, cmap,
 	 */
 	if (interlace) {
 	    ypos += interlaceStep[pass];
-	    while (ypos >= height) {
+	    while (ypos >= rows) {
 		pass++;
 		if (pass > 3) {
 		    return TCL_OK;
@@ -1123,13 +1130,15 @@ GetCode(chan, code_size, flag)
  */
 
 static void
-mInit(string, handle)
-   unsigned char *string;	/* string containing initial mmencoded data */
-   MFile *handle;		/* mmdecode "file" handle */
+mInit(string, length, handle)
+    unsigned char *string;	/* string containing initial mmencoded data */
+    int length;			/* Length of string */
+    MFile *handle;		/* mmdecode "file" handle */
 {
-   handle->data = string;
-   handle->state = 0;
-   handle->c = 0;
+    handle->data = string;
+    handle->length = length;
+    handle->state = 0;
+    handle->c = 0;
 }
 
 /*
@@ -1152,18 +1161,18 @@ mInit(string, handle)
 
 static int
 Mread(dst, chunkSize, numChunks, handle)  
-   unsigned char *dst;	/* where to put the result */
-   size_t chunkSize;	/* size of each transfer */
-   size_t numChunks;	/* number of chunks */
-   MFile *handle;	/* mmdecode "file" handle */
+    unsigned char *dst;	/* where to put the result */
+    size_t chunkSize;	/* size of each transfer */
+    size_t numChunks;	/* number of chunks */
+    MFile *handle;	/* mmdecode "file" handle */
 {
-   register int i, c;
-   int count = chunkSize * numChunks;
+    register int i, c;
+    int count = chunkSize * numChunks;
 
-   for(i=0; i<count && (c=Mgetc(handle)) != GIF_DONE; i++) {
+    for(i=0; i<count && (c=Mgetc(handle)) != GIF_DONE; i++) {
 	*dst++ = c;
-   }
-   return i;
+    }
+    return i;
 }
 
 /*
@@ -1201,6 +1210,10 @@ Mgetc(handle)
     }
 
     do {
+	if (handle->length-- <= 0) {
+	    handle->state = GIF_DONE;
+	    return GIF_DONE;
+	}
 	c = char64(*handle->data);
 	handle->data++;
     } while (c == GIF_SPACE);
@@ -1315,8 +1328,12 @@ Fread(dst, hunk, count, chan)
 	return Mread(dst, hunk, count, (MFile *) chan);
     case 2:
 	handle = (MFile *) chan;
+	if (handle->length <= 0 || (size_t)handle->length < (size_t) (hunk * count)) {
+	    return -1;
+	}
 	memcpy((VOID *)dst, (VOID *) handle->data, (size_t) (hunk * count));
 	handle->data += hunk * count;
+	handle->length -= hunk * count;
 	return (int)(hunk * count);
     default:
 	return Tcl_Read(chan, (char *) dst, (int) (hunk * count));
@@ -1827,7 +1844,7 @@ output(val)
     obuf |= val << obits;
     obits += out_bits;
     while (obits >= 8) {
-	block_out(obuf&0xff);
+	block_out(UCHAR(obuf&0xff));
 	obuf >>= 8;
 	obits -= 8;
     }
@@ -1839,7 +1856,7 @@ output_flush()
 {
     DEBUGMSG(("output_flush\n"));
     if (obits > 0) {
-	block_out(obuf);
+	block_out(UCHAR(obuf));
     }
     block_flush();
 }
